==> falak-0.1.5.tgz: package/framework/compiler/parser/component.js <==
const Tag = require('./tag');
const HtmlCompiler = require('./html-compiler');
const { flatten } = require('./../helpers/flatten');

module.exports = class Component extends Tag {
    /**
     * {@inheritdoc}
     */
    init() {
        this.attributesInBrackets = this.attrs().brackets();

        for (let attribute in this.attributesInBrackets) {
            let attributeValue = this.attributesInBrackets[attribute],
                newAttribute = attribute.toCamelCase();

            delete this.attributesInBrackets[attribute];

            this.attributesInBrackets[newAttribute] = attributeValue;
        }
    }

    /**
     * Get component object 
     * 
     * @returns string
     */
    objectName() {
        return `${this.htmlCompiler.objectName}._c.${this.namespace}`;
    }

    /**
     * Get child name
     */
    childName() {
        if (!this._childName) {
            this._childName = 'c' + random(8);
        }

        return this._childName;
    }

    /**
     * {@inheritDoc}
     */
    build() {
        let componentName = this.childName(),
            objectName = this.objectName(),
            component = this.component,
            componentContent = this.getElement().innerHTML,
            parsedComponentContent = null;

        // check if component has a content passed to it
        // i.e <component-name>Some content</component-name>
        // in that case we need to pass to the component the value of `Some content`
        if (componentContent.length > 0) {
            let contentCompiler = new HtmlCompiler(componentContent);

            contentCompiler.originalCompiler = this.htmlCompiler;

            // contentCompiler.rootNamespace = this.namespace;

            contentCompiler.onComponentLoad(component => {
                this.htmlCompiler.addChild(component.childName(), component.selector);
            });

            contentCompiler.parse();

            parsedComponentContent = `() => {${contentCompiler.parsed}}`;
        }

        this.htmlCompiler.addChild(componentName, this.selector);

        let parentComponent = this.htmlCompiler.objectName;

        let componentAlias,
            componentIsArray = false;

        if (this.attrs().has('as')) {
            componentAlias = this.attrs().forcePull('as');
        }

        let props = {};

        let skippedProperties = [this.variableName];

        for (let attribute in this.attributesInBrackets) {
            let value = this.attributesInBrackets[attribute].replace(/this/g, this.htmlCompiler.objectName);
            props[attribute] = value;
            skippedProperties.push(value);
        }

        let customEvents = this.collectCustomEvents();

        this.clearInvalidAttributes();
        // this.openTag();

        // this.variableNameWillBeUsed();

        let componentInfo = {};

        // componentInfo.parent = this.variableName;

        if (!Is.empty(props)) {
            componentInfo.props = props;
        }

        if (!Is.empty(customEvents)) {
            componentInfo.events = customEvents;
        }

        if (parsedComponentContent) {
            componentInfo.content = parsedComponentContent;
        }

        if (Object.keys(this.events).length) {
            componentInfo.events = this.events;
        }

        if (Object.keys(this.booleanAttributes).length) {
            componentInfo.boolAttrs = this.booleanAttributes;
        }

        if (Object.keys(this.attributes).length) {
            componentInfo.attrs = this.attributes;
        }

        if (Object.keys(this.style).length) {
            componentInfo.style = this.style;
        }

        // it means the component is inside an if statement
        if (this.htmlCompiler.currentState) {
            componentInfo.state = this.htmlCompiler.currentState.state;
        }

        if (this.htmlCompiler.insideForLoop) {
            componentInfo.insideLoop = true;
            componentInfo.index = this.htmlCompiler.currentLoopIndex;
            skippedProperties.push(this.htmlCompiler.currentLoopIndex);

            if (componentAlias) {
                componentIsArray = true;
                this.appendLine(`${parentComponent}.${componentAlias} = []`);
            }
        }

        let secondArgument = '';

        if (!Is.empty(componentInfo)) {
            componentInfo = flatten(componentInfo, {
                skip: skippedProperties,
            });
            secondArgument = `, ${componentInfo}`;
        }

        let componentCode = `this._lc('${componentName}'${secondArgument})`;

        if (componentAlias) {
            if (componentIsArray) {
                this.appendLine(`${parentComponent}.${componentAlias}.push(${componentCode}.component)`);
            } else {
                this.appendLine(`${parentComponent}.${componentAlias} = ${componentCode}`);
            }
        } else {
            this.appendLine(componentCode);
        }

        // this.closeTag();
    }

    /**
     * Collect passed custom events
     */
    collectCustomEvents() {
        let customEvents = {};
        for (let attribute in this.originalAttributes) {
            if (attribute.startsWith('(') && attribute.endsWith(')')) {
                let eventName = `on` + attribute.ltrim('(').rtrim(')').capitalize();

                let functionArguments = this.collectArgumentsOf(this.originalAttributes[attribute]);

                customEvents[eventName] = `function(${functionArguments}) {${this.originalAttributes[attribute]}}`;
            }
        }

        return customEvents;
    }

    /**
     * Get the parameters inside the braces ()
     * 
     * i.e this.update(value1, value2) so we will return value1, value2
     */
    collectArgumentsOf(functionCode) {
        let fullArgumentsList = [];

        functionCode.split(';').forEach(string => {
            string.replace(/\((.*)\)/g, function (matched, argumentsList) {
                // split arguments by semi colon
                argumentsList.split(',').forEach(argument => {
                    argument = argument.trim();

                    if (!fullArgumentsList.includes(argument)) {
                        fullArgumentsList.push(argument);
                    }
                });
            });
        });

        return fullArgumentsList.join(',');
    }
}

==> falak-0.1.16.tgz: package/framework/compiler/parser/component.js <==
const Tag = require('./tag');
const HtmlCompiler = require('./html-compiler');
const { flatten } = require('./../helpers/flatten');

module.exports = class Component extends Tag {
    /**
     * {@inheritdoc}
     */
    init() {
        this.attributesInBrackets = this.attrs().brackets();

        for (let attribute in this.attributesInBrackets) {
            let attributeValue = this.attributesInBrackets[attribute],
                newAttribute = attribute.toCamelCase();

            delete this.attributesInBrackets[attribute];

            this.attributesInBrackets[newAttribute] = attributeValue;
        }
    }

    /**
     * Get component object 
     * 
     * @returns string
     */
    objectName() {
        return `${this.htmlCompiler.objectName}._c.${this.namespace}`;
    }

    /**
     * Get child name
     */
    childName() {
        if (!this._childName) {
            this._childName = 'c' + random(8);
        }

        return this._childName;
    }

    /**
     * {@inheritDoc}
     */
    build() {
        let componentName = this.childName(),
            objectName = this.objectName(),
            component = this.component,
            componentContent = this.getElement().innerHTML,
            parsedComponentContent = null;

        // check if component has a content passed to it
        // i.e <component-name>Some content</component-name>
        // in that case we need to pass to the component the value of `Some content`
        if (componentContent.length > 0) {
            let contentCompiler = new HtmlCompiler(componentContent);

            contentCompiler.originalCompiler = this.htmlCompiler;

            // contentCompiler.rootNamespace = this.namespace;

            contentCompiler.onComponentLoad(component => {
                this.htmlCompiler.addChild(component.childName(), component.selector);
            });

            contentCompiler.parse();

            parsedComponentContent = `() => {${contentCompiler.parsed}}`;
        }

        this.htmlCompiler.addChild(componentName, this.selector);

        let parentComponent = this.htmlCompiler.objectName;

        let componentAlias,
            componentIsArray = false;

        if (this.attrs().has('as')) {
            componentAlias = this.attrs().forcePull('as');
        }

        let props = {};

        let skippedProperties = [this.variableName];

        for (let attribute in this.attributesInBrackets) {
            let value = this.attributesInBrackets[attribute].replace(/this/g, this.htmlCompiler.objectName);
            props[attribute] = value;
            skippedProperties.push(value);
        }

        let customEvents = this.collectCustomEvents();

        this.clearInvalidAttributes();
        // this.openTag();

        // this.variableNameWillBeUsed();

        let componentInfo = {};

        componentInfo.parent = this.variableName;

        if (!Is.empty(props)) {
            componentInfo.props = props;
        }

        if (!Is.empty(customEvents)) {
            componentInfo.events = customEvents;
        }

        if (parsedComponentContent) {
            componentInfo.content = parsedComponentContent;
        }

        if (Object.keys(this.events).length) {
            componentInfo.events = this.events;
        }

        if (Object.keys(this.booleanAttributes).length) {
            componentInfo.boolAttrs = this.booleanAttributes;
        }

        if (Object.keys(this.attributes).length) {
            componentInfo.attrs = this.attributes;
        }

        if (Object.keys(this.style).length) {
            componentInfo.style = this.style;
        }

        // it means the component is inside an if statement
        if (this.htmlCompiler.currentState) {
            componentInfo.state = this.htmlCompiler.currentState.state;
        }

        if (this.htmlCompiler.insideForLoop) {
            componentInfo.insideLoop = true;
            componentInfo.index = this.htmlCompiler.currentLoopIndex;
            skippedProperties.push(this.htmlCompiler.currentLoopIndex);

            if (componentAlias) {
                componentIsArray = true;
                this.appendLine(`${parentComponent}.${componentAlias} = []`);
            }
        }

        let secondArgument = '';

        if (!Is.empty(componentInfo)) {
            componentInfo = flatten(componentInfo, {
                skip: skippedProperties,
            });
            secondArgument = `, ${componentInfo}`;
        }

        let componentCode = `this._lc('${componentName}'${secondArgument})`;

        if (componentAlias) {
            if (componentIsArray) {
                this.appendLine(`${parentComponent}.${componentAlias}.push(${componentCode}.component)`);
            } else {
                this.appendLine(`${parentComponent}.${componentAlias} = ${componentCode}`);
            }
        } else {
            this.appendLine(componentCode);
        }

        // this.closeTag();
    }

    /**
     * Collect passed custom events
     */
    collectCustomEvents() {
        let customEvents = {};
        for (let attribute in this.originalAttributes) {
            if (attribute.startsWith('(') && attribute.endsWith(')')) {
                let eventName = `on` + attribute.ltrim('(').rtrim(')').capitalize();

                let functionArguments = this.collectArgumentsOf(this.originalAttributes[attribute]);

                // customEvents[eventName] = `function(${functionArguments}) {${this.originalAttributes[attribute]}}`;
                customEvents[eventName] = `function() {${this.originalAttributes[attribute]}}`;
            }
        }

        return customEvents;
    }

    /**
     * Get the parameters inside the braces ()
     * 
     * i.e this.update(value1, value2) so we will return value1, value2
     */
    collectArgumentsOf(functionCode) {
        let fullArgumentsList = [];

        functionCode.split(';').forEach(string => {
            string.replace(/\((.*)\)/g, function (matched, argumentsList) {
                // split arguments by semi colon
                argumentsList.split(',').forEach(argument => {
                    argument = argument.trim();

                    if (!fullArgumentsList.includes(argument)) {
                        fullArgumentsList.push(argument);
                    }
                });
            });
        });

        return fullArgumentsList.join(',');
    }
}
